#!/bin/sh -e

TMP="$1"
if [ -z "$TMP" ]; then
  echo "Usage: ./gentypes.sh path/to/tmpdir > path/to/types.h" 1>&2
  exit 1
fi

cat types.h0

if CCITESTRUN "$TMP/gentypes" -DHAVE_LONGLONG gentypes.c; then
  true
elif CCITESTRUN "$TMP/gentypes" gentypes.c; then
  true
else
  echo "failed to compile gentypes.c" 1>&2
  exit 1
fi

echo -n "> have 'size_t'? " 1>&2
if CCITESTOK "$TMP/gensizet.o" -DHAVE_SYS_TYPES_H -DCHECK_SIZET_PRESENT gensizet.c; then
  echo "yes, with sys/types.h" 1>&2
  ARG="-DHAVE_SYS_TYPES_H -DCHECK_SIZET_MATCH"
elif CCITESTOK "$TMP/gensizet.o" -DCHECK_SIZET_PRESENT gensizet.c; then
  echo "yes, without sys/types.h" 1>&2
  ARG="-DCHECK_SIZET_MATCH"
elif CCITESTOK "$TMP/gensizet.o" gensizet.c; then
  echo "no" 1>&2
  ARG=""
else
  echo "failed to compile gensizet.c stage 1" 1>&2
  exit 1
fi

CCITESTRUN "$TMP/gensizet" $ARG -DMT=short -DMTSTR=\"short\" gensizet.c || \
  CCITESTRUN "$TMP/gensizet" $ARG "-DMT=long long" "-DMTSTR=\"long long\"" gensizet.c || \
  CCITESTRUN "$TMP/gensizet" $ARG -DMT=int -DMTSTR=\"int\" gensizet.c || \
  CCITESTRUN "$TMP/gensizet" $ARG -DMT=long -DMTSTR=\"long\" gensizet.c || \
  exit 1



if CCITESTOK "$TMP/inline.o" -DFCL_INLINE="static __inline__ __attribute__((always_inline))" inline.c; then
  echo ""
  echo "#define FCL_INLINE static __inline__ __attribute__((always_inline))"
elif CCITESTOK "$TMP/inline.o" -DFCL_INLINE="static inline" inline.c; then
  echo ""
  echo "#define FCL_INLINE static inline"
fi

if CCITESTOK "$TMP/noreturn.o" -DFCL_NORETURN="__attribute__((__noreturn__))" noreturn.c; then
  echo ""
  echo "#define FCL_NORETURN __attribute__((__noreturn__))"
elif CCITESTOK "$TMP/noreturn.o" -DFCL_NORETURN="_Noreturn" noreturn.c; then
  echo ""
  echo "#define FCL_NORETURN _Noreturn"
else
  echo "#define FCL_NORETURN"
  echo "#define FCL_NO_NORETURN"
fi

cat types.h1

